#pragma rtGlobals=1		// Use modern global access method.

//This is window Procedure_TripleExpFit

//April 2001:  Extended to include  Simplex fits




//CurveFit/G dblexp PoolCmCon1(0.2,0.6) /W=PoolCmCon2 /D 
//FuncFit/H="00000001" Triple FitPar PoolCmCon1(0.2,0.6) /W=PoolCmCon2 /D 


Macro FitTripleE (ParName,t_end,Flag)
	string ParName
	Variable t_end,flag
	//A triple exponential fit - one rising and two decaying exps. -  to the curve on which the X-cursor sits
	//A cursor has to be put onto a curve in the top window!
	//ParName is the name of a wave which is created by the macro and which contains
	// the fit parameters (called FitPar; see meaning of the parameters below )
	// Fitting is done between cursor and t_end
	//If flag is 2. the fastest component will be fixed to zero amplitude: 2 component fit
	//otherwise  threecompnent fit
	//  weights:  that  the first fifth of the  fitting range has 5 time the weigtht  as the rest 
	// initial guesses:  A fixed pattern based on the x-range (Tot) and the y-range (Y_tot)
	// of the data has been programmed (see below). It is readily changed
	
	//FitPar[0] is the asymptotic Y-value; FitPar[1],FitPar[2]  are amp and Tau  of fastest comp.
	//FitPar[3],FitPar[4]  correspondingly of second compnent
	//FitPar[5],FitPar[6]  correspondingly of third compnent
	//FitPar[7]  is time 0, which is being fixed during the fitting

	
	string  YName
	Variable t_start, t_wave, Y_tot,tot
	Make/N=8/D/O $ParName
	V_FitMaxiters = 60
	
	YName = CsrWave (A)	//Name of  wave that the cursor is on
	Duplicate/O $YName, Weightf
	Weightf = x
	t_wave = Weightf[(numpnts(Weightf)-1)]
	t_start = xCsr(A)
	if  (t_end > t_wave)
		Abort  "t_end  later than wave_end"
	endif
	tot = t_end - t_start
	Y_tot = 1.5* ($YName (t_start) - $YName (t_end))
	Weightf = 1
	Weightf (t_start,t_start+tot/5) = 5

		//initial guesses:
	$ParName[0] = $YName (t_start) - Y_tot	//Baseline
	$ParName[1] = 0.30*Y_tot 				//Amp of fastest comp.
	$ParName[2] = 0.01*tot					//Tau of fastest comp.
	$ParName[3] = 0.3*Y_tot 					//Amp of interm. comp.
	$ParName[4] =0.1*tot						//Tau of interm. comp.
	$ParName[5] =0.4*Y_tot 					//Amp of slow comp.
	$ParName[6] =2*tot						//Tau of slow comp.
	$ParName[7] =t_start
	//Print $ParName[0]
	If  (Flag ==2)
		$ParName[1] = 0
		FuncFit/H="01100001" Triple $ParName $YName(t_start,t_end) /W=Weightf /D
		Print "2comp. Fit to:",  YName, "  from   ", t_start, "to  ", t_end,"sec"
		Print  "Tau's:	", 1000*$ParName[4],1000*$ParName[6],"	msec"
		Print  "Amps:	", -1e15*$ParName[3], -1e15*$ParName[5],"	fF"

	ELSE
		FuncFit/H="00000001" Triple $ParName $YName(t_start,t_end) /W=Weightf /D
		Print "3comp. Fit to:",  YName, "  from   ", t_start, "to  ", t_end,"sec"
		Print  "Tau's:	", 1000*$ParName[2], 1000*$ParName[4],1000*$ParName[6],"	msec"
		Print  "Amp's:	", -1e15*$ParName[1], -1e15*$ParName[3], -1e15*$ParName[5],"	fF"
	Endif
end



Function/D Triple (Parwave, x)
	wave/D Parwave
	Variable/D x
	//Parwave[0] is final value; Parwave[1],Parwave[2]  are amp and timeconstant  of fastest comp.
	//Parwave[3],Parwave[4]  correspondingly of second compnent
	//Parwave[5],Parwave[6]  correspondingly of third compnent
	//Parwave[7]  is time 0, which has to be fixed during the fitting
	If  (Parwave[2] == 0)
		Parwave[2] = 1e-4
	ENDIF
	If  (Parwave[4] == 0)
		Parwave[4] = 1e-4
	ENDIF
	If  (Parwave[6] == 0)
		Parwave[6] = 1e-4
	ENDIF
	Variable result
	result = Parwave[0] + Parwave[1]*exp(-(x-Parwave[7])/Parwave[2])
	result  += Parwave[3]*exp(-(x-Parwave[7])/Parwave[4])
	result  += Parwave[5]*exp(-(x-Parwave[7])/Parwave[6])

	return result
end

Function/D DLorentz (Parwave, x)
	wave/D Parwave
	Variable/D x
	//Parwave[0] is 1st amplitude; Parwave[1] is first corner frequency
	//Parwave[2] is 2nd amplitude; Parwave[3] is second corner frequency
	If  (Parwave[1] == 0)
		Parwave[1] = 1e-4
	ENDIF
	If  (Parwave[3] == 0)
		Parwave[3] = 1e-4
	ENDIF
	If  (Parwave[1] < 0)
		Parwave[1] *=-1
	ENDIF
	If  (Parwave[3] < 0)
		Parwave[3] *=-1
	ENDIF
	If  (Parwave[0] < 0)
		//Parwave[0] *=-1
	ENDIF
	If  (Parwave[2] < 0)
		//Parwave[2] *=-1
	ENDIF
	Variable result
	result = Parwave[0]/(1 + (x/ Parwave[1])^2)
	result +=Parwave[2]/(1 + (x/ Parwave[3])^2)
	return result
end


Function/D MiniSpec (Parwave, x)
	wave/D Parwave
	Variable/D x
	//Parwave[0] is 1st amplitude; Parwave[1] is first corner frequency
	// Parwave[3] is second corner frequency
	If  (Parwave[1] == 0)
		Parwave[1] = 1e-4
	ENDIF
	If  (Parwave[3] == 0)
		Parwave[3] = 1e-4
	ENDIF
	If  (Parwave[1] < 0)
		Parwave[1] *=-1
	ENDIF
	If  (Parwave[3] < 0)
		Parwave[3] *=-1
	ENDIF
	If  (Parwave[0] < 0)
		//Parwave[0] *=-1
	ENDIF
	Variable result, factor
	factor = 1 + x^2*(Parwave[1]^(-2)+Parwave[3]^(-2))
	Factor  += x^4*Parwave[1]^(-2)*Parwave[3]^(-2)
	result = Parwave[0]*(1/Parwave[1]-1/Parwave[3])^2/factor/(4*Pi^2)
	return result
end

Macro FitWholeMini (ParName,t_end,Flag)
	string ParName
	Variable t_end,flag
	//A triple exponential fit  to the curve on which the X-cursor sits
	//A cursor has to be put onto a curve in the top window!  select the point to be about 1/5th  Y_max
	//ParName is the name of a wave which is created by the macro and which contains
	// the fit parameters (called FitPar; see meaning of the parameters below )
	// Fitting is done between cursor and t_end
	//If flag is 2. the fastest component will be fixed to zero amplitude: 2 component fit
	//otherwise  threecompnent fit
	//  weights:  that  the first fifth of the  fitting range has 5 time the weigtht  as the rest 
	// initial guesses:  A fixed pattern based on the x-range (Tot) and the y-range (Y_tot)
	// of the data has been programmed (see below). It is readily changed
	
	//FitPar[0] is the asymptotic Y-value; FitPar[1],FitPar[2]  are amp and Tau  of fastest comp.
	//FitPar[3],FitPar[4]  correspondingly of second compnent
	//FitPar[5],FitPar[6]  correspondingly of third compnent
	//FitPar[7]  is time 0, which is being fixed during the fitting

	
	string  YName
	Variable t_start, t_wave, Y_tot,tot
	Make/N=8/D/O $ParName
	//V_FitMaxiters = 60
	
	YName = CsrWave (A)	//Name of  wave that the cursor is on
	Duplicate/O $YName, Weightf
	Weightf = x
	t_wave = Weightf[(numpnts(Weightf)-1)]
	t_start = xCsr(A)
	if  (t_end > t_wave)
		Abort  "t_end  later than wave_end"
	endif
	tot = t_end - t_start
	Y_tot = 1.5* ($YName (t_start) - $YName (t_end))
	Weightf = 1
	Weightf (t_start,t_start+tot/5) = 5

		//initial guesses:
	$ParName[0] = $YName (t_start) - Y_tot	//Baseline
	$ParName[1] = -5*Y_tot 				//Amp of fastest comp.
	$ParName[2] = 0.0003				//Tau of fastest comp.
	$ParName[3] =5* 0.8*Y_tot 					//Amp of interm. comp.
	$ParName[4] =0.002						//Tau of interm. comp.
	$ParName[5] =5*0.2*Y_tot 					//Amp of slow comp.
	$ParName[6] =0.01						//Tau of slow comp.
	$ParName[7] =t_start
	//Print $ParName[0]
	If  (Flag ==2)
		$ParName[1] = 0
		//Overwrite initial guesses
		$ParName[3] =-5*Y_tot 					//Amp of interm. comp.
		$ParName[4] =0.0002						//Tau of interm. comp.
		$ParName[5] =5* Y_tot 					//Amp of slow comp.
		$ParName[6] =0.004						//Tau of slow comp.

		FuncFit/H="01100001" Triple $ParName $YName(t_start,t_end) /W=Weightf /D
		Print "2comp. Fit to:",  YName, "  from   ", t_start, "to  ", t_end,"sec"
		Print  "Tau's:	", 1000*$ParName[4],1000*$ParName[6],"	msec"
		Print  "Amps:	", -1e15*$ParName[3], -1e15*$ParName[5],"	fF"

	ELSE
		FuncFit/H="00000001" Triple $ParName $YName(t_start,t_end) /W=Weightf /D
		Print "3comp. Fit to:",  YName, "  from   ", t_start, "to  ", t_end,"sec"
		Print  "Tau's:	", 1000*$ParName[2], 1000*$ParName[4],1000*$ParName[6],"	msec"
		Print  "Amp's:	", -1e15*$ParName[1], -1e15*$ParName[3], -1e15*$ParName[5],"	fF"
	Endif
end


Macro FitRelease (ParName,t_end,Flag)
	string ParName
	Variable t_end,flag
	
	//This is rudimentary, presently confined to hand-fits;   28.Feb.01
	//  It turned out that it is difficult to fit rising phases of rates
	
	//A 7 component  exponential fit - see below -  to the curve on which the X-cursor sits
	//A cursor has to be put onto a curve in the top window to mark the start of the fitting; this 
	//should be the true start of the stimulation, e.g. the point of half rise of the Ca-current
	//ParName is the name of a wave which is created by the macro and which contains
	// the fit parameters (called FitPar; see meaning of the parameters below )
	// Fitting is done between cursor and t_end
	//If flag is 0:	allparameters free except for time of start
	//			1:	normal:  fact (rise time expansion factor) is fixed, other timing is free
	//			4:	all time constants are frozen
	//  weights:  that  the first fifth of the  fitting range has 5 time the weigtht  as the rest 
	// initial guesses:  A fixed pattern based on the x-range (Tot) and the y-range (Y_tot)
	// of the data has been programmed (see below). It is readily changed
	
	//We set up the  two component (fast and slow) plus steady state according to the following plan:
	// The steady state component for simplicity is supposed to rise with the decay time of the slow component.  
	// The two component start  and end with zero, each consists of three exponentials:
	//start=0:   A0i + A1i + A2i  = 0  -->  A0i =  -( A1i + A2i)
	//end=0:   we work with decaying exponentials except for the constant term
	//Time constants are  Tau0i, Tau1i and Tau2i,  where i stands for 1 or 2
	//Restriction:  Tau02 and Tau12 are assumed to be multiples of  Tau01 and Tau11, such that the rising phase
	//of the slow component is just a time-scaled version of that of the fast phase; the scaler is 'fact'
	// y(x) =  A3   -  (A11+A21)*exp(-x/tau01)  +  A11*exp(-x/tau11) + A21*exp(-x/tau21)
	// y(x) =       +-  (A12+A22)*exp(-x/(fact*tau01))  +  A12*exp(-x/(fact*tau11)) + A22*exp(-x/tau22)
	//
	//We, thus have 11 parameters for the  fitparameterwave  A3, A11,A21, A12,A22,tau01,tau11,tau21,fact,tau22,t_start
	//FitPar[0] is the asymptotic Y-value A3
	//FitPar[1],FitPar[2]  are amps of the first comp. A11 and A21
	//FitPar[3],FitPar[4]  are amps of the second comp. A12 and A22
	//FitPar[5] to FitPar[7]  are Taus of the first component : tau01,tau11,tau21
	//FitPar[8] and FitPar[9]  are fact and Tau22
	//FitPar[10]  is time 0, which is being fixed during the fitting
	//
	//For the fitting,  we assume A3 to represent a constant component; for evaluation of the contributions (areas)
	//of the two components, however, we assume the 'refilling rate' to rise with  Tau22 to a steady state value of A3
	//Then,  the area under the fast component is given by:   A11*(Tau11-Tau01) + A21*(Tau21-Tau01)
	//the area under the slow component:   A12*fact*(Tau11-Tau01) + A22*(Tau22-fact*Tau01) + A3*Tau22
	//...this is readily derived by taking the total area as the sum of  two components plus a  refilling component,
	//as defined above

	
	string  YName
	Variable t_start, t_wave, Y_tot,tot
	Make/N=11/D/O $ParName
	//V_FitMaxiters = 60
	
	YName = CsrWave (A)	//Name of  wave that the cursor is on
	Duplicate/O $YName, Weightf,Testf
	Weightf = x
	t_wave = Weightf[(numpnts(Weightf)-1)]
	t_start = xCsr(A)
	if  (t_end > t_wave)
		Abort  "t_end  later than wave_end"
	endif
	tot = t_end - t_start
	Wavestats/Q/R=(t_start,t_end)  $YName
	Y_tot = 20*V_max
	Weightf = 1
	Weightf (t_start,t_start+tot/5) = 5

		//initial guesses:
	$ParName[0] = 0//$YName (t_end) 	//Baseline is set to final value   A3
	$ParName[1] = -1.23*Y_tot ;$ParName[2] = Y_tot	//Amp of fast comp.  A11, A21
	//$ParName[3] = -3/2*Y_tot/5 ;$ParName[4] =Y_tot/5	//Amp of slow comp. A12, A22
	$ParName[3] = 0 ;$ParName[4] =0	//Amp of slow comp. A12, A22
	$ParName[5] = 0.01*tot			//DelayTau of first comp.  0.3% of total time, 	Tau01
	$ParName[6] = 0.02*tot			//RiseTau of first comp.  1% of total time		Tau11
	//$ParName[7] = 0.03*tot			//DecayTau of first comp.  3% of total time		Tau21
	$ParName[7] = 0.0012	//DecayTau of first comp.  3% of total time		Tau21
	$ParName[8] = 2					//Slowdown factor of rising phase of second component
	$ParName[9] = 5*	$ParName[7]	//DecayTau of slow comp. 5 times that of fast one	 Tau22
	$ParName[10] =t_start
	//Print $ParName[0]
	testf(t_start,t_end) = ReleaseFunc ($ParName, x)	//for test purposes or handfit
	
end
	
	If  (Flag ==2)
		$ParName[1] = 0
		FuncFit/H="01100001" Triple $ParName $YName(t_start,t_end) /W=Weightf /D
		Print "2comp. Fit to:",  YName, "  from   ", t_start, "to  ", t_end,"sec"
		Print  "Tau's:	", 1000*$ParName[4],1000*$ParName[6],"	msec"
		Print  "Amps:	", -1e15*$ParName[3], -1e15*$ParName[5],"	fF"

	ELSE
		FuncFit/H="00000001" Triple $ParName $YName(t_start,t_end) /W=Weightf /D
		Print "3comp. Fit to:",  YName, "  from   ", t_start, "to  ", t_end,"sec"
		Print  "Tau's:	", 1000*$ParName[2], 1000*$ParName[4],1000*$ParName[6],"	msec"
		Print  "Amp's:	", -1e15*$ParName[1], -1e15*$ParName[3], -1e15*$ParName[5],"	fF"
	Endif
end



Function/D ReleaseFunc (Parwave, x)
	wave/D Parwave
	Variable/D x
	//		see definition of parameters in the comments of Macro FitRelease
	If  (Parwave[5] == 0)
		Parwave[5] = 1e-5
	ENDIF
	If  (Parwave[6] == 0)
		Parwave[6] = 1e-5
	ENDIF
	If  (Parwave[7] == 0)
		Parwave[7] = 1e-5
	ENDIF
	If  (Parwave[9] == 0)
		Parwave[9] = 1e-5
	ENDIF
	
	x -= Parwave[10]
	
	Variable result
	result =  Parwave[0]   -  (Parwave[1]+Parwave[2])*exp(-x/Parwave[5])  //A3 + 1st delay component
	result +=  Parwave[1]*exp(-x/Parwave[6]) + Parwave[2]*exp(-x/Parwave[7])//rise and fall of 1st comp.
	result -=  (Parwave[3]+Parwave[4])*exp(-x/(Parwave[8]*Parwave[5])) //2nd delay component
	result +=  Parwave[3]*exp(-x/(Parwave[8]*Parwave[6])) + Parwave[4]*exp(-x/Parwave[9])//the rest
	return result
end



Macro FitDoublelorentz(ParName,f_end,Flag)
	string ParName
	Variable f_end,flag
	//A double Lorentzian fit  to the curve on which the X-cursor sits
	//A cursor has to be put onto a curve in the top window!
	//ParName is the name of a wave which is created by the macro and which contains
	// the fit parameters (called FitPar; see meaning of the parameters below )
	// Fitting is done between cursor and f_end
	//If flag is 1. the second component will be fixed to zero amplitude: 1 component fit
	//If flag =2  twocompnent fit;  if 0:  Fit to the spectrum of a mini (according to Segal
	//et al  Biophys. J. 47, p. 184
	//  weights:  no weights so far
	// initial guesses:  A fixed pattern based on the x-range (Tot) and the y-range (Y_tot)
	// of the data has been programmed (see below). It is readily changed
	
	//Parwave[0] is 1st amplitude; Parwave[1] is first corner frequency
	//Parwave[2] is 2nd amplitude; Parwave[3] is second corner frequency
	
	string  YName
	Variable f_start, f_wave, Y_tot,tot
	Make/N=4/D/O $ParName
	//V_FitMaxiters = 120
	
	YName = CsrWave (A)	//Name of  wave that the cursor is on
	Duplicate/O $YName, Weightf
	Weightf =x		//To get the scaling of the x-wave
	f_wave = Weightf[(numpnts(Weightf)-1)]	//The frequency of the last point in the spec
	f_start = Weightf( xCsr(A)	)	//The frequency of the first point in the fitting range
	If (F_start ==0)
		F_start = deltax($YName)/3
	endif
	if  (f_end > f_wave)
		Abort  "t_end  later than wave_end"
	endif
	tot = f_end - f_start
	Y_tot = $YName (f_start) 
	Weightf = (F_start*2 + x)^3
	//Weightf (t_start,t_start+tot/5) = 5

		//initial guesses:
	$ParName[0] =  2*Y_tot			//First asymptote
	$ParName[1] = 2*F_start	//First corner frequency
	$ParName[2] = 0.01*Y_tot		//Secondt asymptote
	$ParName[3] = 20*F_start 	//Second  corner frequency
	//Print $ParName[0],$ParName[1],$ParName[2],$ParName[3]
	//MiSpec  = MiniSpec($ParName,x)
	If  (Flag ==1)
		$ParName[2] = 0
		FuncFit/H="0011" DLorentz  $ParName $YName(f_start,f_end) /W=Weightf /D
		Print "1comp. lorentzian Fit to:",  YName, "  from   ", f_start, "to  ", f_end,"sec"
		Print  "Tau:	", 1000/(2*Pi*$ParName[1]),"	msec"
		Print  "Amp:	", $ParName[0],"	Amp^2/Hz"
	Endif

	If  (Flag ==2)
		FuncFit DLorentz $ParName $YName(f_start,f_end) /W=Weightf /D
		Print "2comp. Lorentzian Fit to:",  YName, "  from   ", f_start, "to  ", f_end,"sec"
		Print  "Tau's:	", 1000/(2*Pi*$ParName[1]), 1000/(2*Pi*$ParName[3]),"	msec"
		Print  "Frequ.s:	", $ParName[1], $ParName[3],"	Hertz"
		Print  "Amp's:	", $ParName[0], $ParName[2],"	Amp^2/Hz"
	Endif
	If  (Flag ==0)
		$ParName[0]  /= (1/$Parname[1]-1/$Parname[3])^2*4*Pi^2/2e3
		//MiSpec  = MiniSpec($ParName,x)
		FuncFit/H="0010" MiniSpec $ParName $YName(f_start,f_end) /W=Weightf /D
		Print "MiniSpectrum Fit to:",  YName, "  from   ", f_start, "to  ", f_end,"sec"
		Print  "Tau's:	", 1000/(2*Pi*$ParName[1]), 1000/(2*Pi*$ParName[3]),"	msec"
		Print  "Frequ.s:	", $ParName[1], $ParName[3],"	Hertz"
		Print  "Amp:	", $ParName[0],"	Amp^2*Nu  or  Nu=", $ParName[0]/(3.11e-11)^2
		Print "							Frequency in terms of full mini size"
	Endif
	
end



Macro LogFitCapac (ParName,numberBins,Sig0,capac,smoothNo,flag,DFlag)
	string ParName
	Variable numberBins,Sig0,capac,smoothNo,flag,DFlag
	//This Macro comes from Procedure File:  TripleE_Fit_Proc and is fully explained
	//in the Notebook of Experiment  'LogBinning'
	//A triple exponential fit  after log-transformation to the curve on which the X-cursor sits
	//A cursor has to be put onto a curve in the top window!
	//ParName is the name of a wave which is created by the macro and which contains
	// the fit parameters ( see meaning of the parameters below )
	// Fitting is done on the whole log-transformed curve, which is restricted to the section
	//between cursors on the original curve;  the first cursor is taken as time zero for log-
	//transformation; numberBins is the number of bins per decade
	//Sig0 is the rms-vaklue of background noise; capac is the capacitance of a granule
	//If flag is 2. ....    : 2 component fit
	//otherwise  threecompnent fit;  DFlag = 1 --> Display      0 ----> no new display
	//  weights ar determined as the inverse of the expected variance
	// initial guesses:  A fixed pattern based on the x-range (Tot) and the y-range (Y_tot)
	// of the data has been programmed (see below). It is readily changed
	
	//ParName[0] is the asymptotic Y-value of the transformed curve - usually fixed to 0;
	// ParName[1],ParName[2]  are amp (inFarads) and Tau (in sec) of fastest comp.
	// ParName[3],ParName[4]  are amp (inFarads) and Tau (in sec) of fsecond comp.
	// ParName[5],ParName[6]  are amp (inFarads) and Tau (in sec) of sustained comp.

	
	string  YName, TName, WName,FName
	Variable t_start, t_end, Y_tot,Tot,npoints
	YName = CsrWave (A)	//Name of  wave that the cursor is on
	TName = "T"+ YName
	WName = "W"+ YName
	FName =  "fit_" + TName
	Make/N=7/D/O $ParName
	Make/N=(10*numberBins)/D/O $TName	//the result wave; at the moment just long 
												//enough to accommodate at least  10 decades


	t_start = xCsr(A)
	t_end = xCsr(B)
	if  (t_end <= t_start)
		t_start = xCsr(B)
		t_end = xCsr(A)
	endif
	tot = t_end - t_start
	Y_tot =  ($YName (t_start) - $YName (t_end))

		//initial guesses:
	$ParName[0] = 0
	$ParName[1] = 0.3*Y_tot 	//these parameters relate to the original capacitance trace
	$ParName[2] = 0.01*tot
	$ParName[3] = 0.3*Y_tot 
	$ParName[4] =0.08*tot
	$ParName[5] =0.4*Y_tot 
	$ParName[6] =2*tot
	//Print $ParName[0]
	
		//transform the curve

	nPoints =  LogTrans ($YName, $TName,t_start,t_end,numberBins)
	print nPoints
	
		//and smooth

	Smooth/B SmoothNo, $TName
	

		//make the weigth function:  we assume that Variance is the sum of  2*background variance
		//plus Variance from statistics of fusion events
		
	Duplicate/O $TName, $WName
	$WName= 1/(2*Sig0^2 + capac*$TName*ln(10)/Numberbins)	//the last two terms are neede to 
										//undo the scaling applied within the LogTrans-FUNCTION




	If  (Flag ==2)
		$ParName[1] = 0
		FuncFit/H="1100000" TripleLogExp $ParName $TName /W=$WName/D
		Print "2comp. Fit to:",  YName, "  from   ", t_start, "to  ", t_end,"sec"
		Print  "Tau's:	", 1000*$ParName[4],1000*$ParName[6],"	msec"
		Print  "Amps:	", -1e15*$ParName[3], -1e15*$ParName[5],"	fF"

	ELSE
		FuncFit/H="1000000" TripleLogExp $ParName $TName /W=$WName/D
		Print "3comp. Fit to:",  YName, "  from   ", t_start, "to  ", t_end,"sec"
		Print  "Tau's:	", 1000*$ParName[2], 1000*$ParName[4],1000*$ParName[6],"	msec"
		Print  "Amp's:	",1e15*$ParName[1], 1e15*$ParName[3], 1e15*$ParName[5],"	fF"
	Endif

	If  (DFlag ==1)
		//Display
		Display $TName,$FName
		AppendToGraph/R $WName
		SetAxis left 0,4e-13  ;DelayUpdate
		SetAxis right 0,5e+27 
		Legend/N=text0/F=0/A=MC
		ShowInfo
		ModifyGraph lstyle($FName)=3,rgb($FName)=(39321,39319,1)
		ModifyGraph rgb($WName)=(1,39321,19939)
	Endif

end



Function  LogTrans (SourceW, ResW,t_s,t_e,n)
	wave SourceW, ResW		//sourcewave, resulting waves x-scaled according to logt
	Variable t_s,t_e,n			//start and end time values in source wave,
								//number of points per decade
								//The function returns the number of points in the result wave
	//and it returns the resultwave with x-scaling according to  logt , but  y-values according to 
	//a ln(t) transformation.  Its maximum is at  log tau, and, for a single exponential 
	// the max-value is  equal to    the relaxation amplitude of the Sourcewave * exp(-1)
	//
	
	//Strategy:  We define the start-time and end-time of the first bin, get the corresponding y-values
	//by interpolation of  the closest sourcewave-values, and take the difference between these
	//as the first  y-value of the resultwave; we then recursively update the bin-limits, and get new 
	//y-values as before by extrapolation;  we calculate the x-scaling of the resultwave as  1/n
	
	// the start value (t_s) in the sourcewave is taken a s time 0; therefore, we will shift below the points 
	//with index is_s to  is_e of the source wave to positions 0, ff. These will be given by:
	
	Variable is_s =  x2pnt(SourceW,t_s), is_e = x2pnt(SourceW,t_e)
	
	//The  factor by which  the start-time of a given transformed bin is offset from the previous start is:
	
		Variable factor = exp (2.3025/n)

	//the first  bin of the transformed  wave should correspond to an interval about 0.5 decades 
	//down from the sample interval (si); so we set the initial time t_in of the first bin (in units of 
	//sample interval) to 

		Variable t_in = sqrt(0.1)		
  
	//the finaltime t_fi of this transformed bin should be:

		Variable t_fi =  t_in*factor		
		
	 // The starting value for the x-scaling
	//of the  transformed wave should therefore be:    log ( deltax(SourceW)*(t_in + t_fi)/2)
	//the increment  of x-scaling in that wave is  log factor  =  2.3025/n/2.3025 = 1/n
	//Note, that we essentially have a logt - transformation, not a lnt - transformation
	
	Duplicate/O SourceW  HelpW
	HelpW = SourceW[is_e]
	HelpW[0,(is_e - is_s)] = SourceW[p+is_s]	//this is now a shifted version of the sourcewave
	SetScale/P x (log( deltax(SourceW)*(t_in + t_fi)/2)),(1/n),"", ResW
	
	Variable i= 0,p1 = trunc(t_in),p2 = trunc(T_fi), y1,y2
		//First calculate the y-values of the first point of the transformed curve
	y2 =  HelpW[p2]*(1- T_fi + p2) + HelpW[p2+1]*( T_fi - p2)	//this is the interpolated y-value at the end point
	y1 =  HelpW[p1]*(1- T_in + p1) + HelpW[p1+1]*( T_in - p1)	//this is the interpolated y-value at the start point

		//now do all the other  points
	do
		ResW[i] =  Y2 - Y1
		y1 = y2				//the new strart time of the transformed bin
		T_fi = T_fi*factor	// update the end of the bin
		p2 = trunc(T_fi)
		y2 =  HelpW[p2]*(1- T_fi + p2) + HelpW[p2+1]*( T_fi - p2)
		i +=1
	while  (T_fi <= (is_e - is_s))
	Redimension/N=(i)  ResW
	ResW *= n	/ln(10)		//scaling such that max-amp = Ampl. of sourcewave * exp(-1)
							//this is the same y-scaling as Sigworth's ln-transform
	KillWaves HelpW
	return i
end
			
	

Function/D TripleLogExp (ParWave,x)
	Wave/D  ParWave
	Variable/D x
	//Calculates the lnt - transformed  curve of a  triple exponential, taking log(t)
	//as x-values
	//Parwave[0] is final value, usually 0;    x is logt (with t in the same units as the time constants)
	// Parwave[1],Parwave[2]  are amp and timeconstants  of fastest comp.
	//Parwave[3],Parwave[4]  correspondingly of second compnent
	//Parwave[5],Parwave[6]  correspondingly of third compnent
	//the reults will have to be scaled according to 
	
	Variable/D result, x1
	x1 = x*ln (10)	//to be able to use log-scale data and the Sine and Sigworth's formula
	result = Parwave[0] + Parwave[1]/Parwave[2]*exp(x1-exp(x1)/Parwave[2])
	result  += Parwave[3]/Parwave[4]*exp(x1-exp(x1)/Parwave[4])
	result  += Parwave[5]/Parwave[6]*exp(x1-exp(x1)/Parwave[6])
	
	return result

end




Function  SinglePoolRelRate  (CaCurrent,alfa, beta,gamma,coop,Poolsize,p_start,p_end,rho,delay)
Wave CaCurrent
variable  alfa , beta , gamma , coop,Poolsize,p_start,p_end,rho,delay

	Duplicate/O CaCurrent , Stimf
	stimf[p_start, p_end] = CaCurrent[p- delay]
	stimf *= rho 
	clip1 (stimf, 0)		//clip stimf to a maximum value of zero

	cadepend1 (Stimf,alfa, beta,gamma,coop,Poolsize,p_start,p_end)	//returns waves, including exo (the 
				//cumulative exocytosis) and  timewave (the corresponding time points)

	Duplicate/O exo , SimRate
	Duplicate/O Timewave, SimX
	Differentiate Simrate,SimX
	SimRate /=SimX
	SimRate /=1000	//this should be in events/msec

	
end



Function  TwoPoolRelRate  (CaCurrent,ParWave,p_start,p_end)
Wave CaCurrent,ParWave	//
variable  p_start,p_end				//a steady state fixed component
	//returns  SimRate and Timewave  as the simulation Y- and X- waves

	Duplicate/O CaCurrent , Stimf
	Variable delay = ParWave[7]/ deltax(CaCurrent), rho = ParWave[6], rho2 = ParWave[8]
	//stimf[p_start, p_start+delay] = 0	//We do not want earlier points to matter
	stimf = 0	//We do not want earlier points to matter
	stimf[p_start+delay, p_end] = CaCurrent[p- delay]
	//print "delay-points",delay
	stimf *= rho 
	clip1 (stimf, 0)		//clip stimf to a maximum value of zero
//Display stimf
	cadepend1 (Stimf,ParWave[0], ParWave[1],ParWave[2],ParWave[3],ParWave[4],p_start,p_end)	
		//makes and returns waves, including 'exo' (the cumulative exocytosis) and  timewave (its X-values)
	Duplicate/O exo , SimRate
	Duplicate/O Timewave, SimX	//Save the x-wave  of the fast component as our result-timewave
	IF (ParWave[5] != 0)	//If there is a second component
		Stimf *= (rho2/rho)
		//Print "working on 2nd component"
		cadepend1 (Stimf,ParWave[0], ParWave[1],ParWave[2],ParWave[3],ParWave[5],p_start,p_end)
		SimRate = SimRate + interp(Simx[p],TimeWave,exo)	//increment SimRate by the interpolated value
	EndIf
	Duplicate/O SimRate BaseLine
	wavestats/Q BaseLine
	BaseLine /= V_max		// This is a normalized version of the cumulative release
	Duplicate/O SimX,  Timewave //restore the first timewave
	Differentiate Simrate,SimX	//destroys SimX
	SimRate /=SimX
	SimRate /=1000	//this should be in events/msec,  with Timewave as the correct  timebase
	SimRate += ParWave[9]*BaseLine // add the  steady state component    in events/msec
		//as a product of the steady state rate and the  normalized cumulative release

	end
	

Function AssignParsToMatrix(ParWave, DestMatrix,Firstindex,Flag)
	Wave ParWave, DestMatrix
	Variable Firstindex,Flag

	NVAR FitP1 = FitP1
	NVAR FitP2 = FitP2
	NVAR FitP3 = FitP3
	AssignIndices (flag)
		DestMatrix[FirstIndex][0] = Parwave[FitP1]
		DestMatrix[FirstIndex][1] = Parwave[FitP2]
		DestMatrix[FirstIndex][2] = Parwave[FitP3]
End

Function AssignMatValuesToPars(ParWave, Matrix,Firstindex,Flag)
	Wave ParWave, Matrix
	Variable Firstindex,Flag

	NVAR FitP1 = FitP1
	NVAR FitP2 = FitP2
	NVAR FitP3 = FitP3
	AssignIndices (flag)
		Parwave[FitP1] = Matrix[FirstIndex][0] 
		Parwave[FitP2] = Matrix[FirstIndex][1] 
		Parwave[FitP3] = Matrix[FirstIndex][2] 
End


Function AssignParsToWave(ParWave, ThisWave,Flag)
	Wave ParWave, ThisWave
	Variable flag

	NVAR FitP1 = FitP1
	NVAR FitP2 = FitP2
	NVAR FitP3 = FitP3
	AssignIndices (flag)
		Thiswave[0] = Parwave[FitP1]
		Thiswave[1] = Parwave[FitP2]
		Thiswave[2] = Parwave[FitP3]
End


Function AssignWaveToPars( ThisWave,ParWave,Flag)
	Wave ThisWave,ParWave
	Variable flag

	NVAR FitP1 = FitP1
	NVAR FitP2 = FitP2
	NVAR FitP3 = FitP3
	AssignIndices (flag)
		Parwave[FitP1] = Thiswave[0] 
		Parwave[FitP2] =Thiswave[1]  
		Parwave[FitP3] = Thiswave[2] 
End

Function AssignIndices (flag)
	Variable  flag

	NVAR FitP1 = FitP1
	NVAR FitP2 = FitP2
	NVAR FitP3 = FitP3
	SVAR Fit1 = Fit1
	SVAR Fit2 = Fit2
	SVAR Fit3 = Fit3
	If (flag==0)
		Fit1 = "Manual"; Fit2 = "Manual";Fit3 = "Manual"
	Endif
	If (flag==1)
		FitP1 = 4; FitP2 = 6;FitP3 = 7
		Fit1 = "Pool1"; Fit2 = "Rho";Fit3 = "Delay"
	Endif
	If (flag==2)
		FitP1 = 4; FitP2 = 5;FitP3 = 6
		Fit1 = "Pool1"; Fit2 = "Pool2";Fit3 = "Rho"
	Endif
	If (flag==3)
		FitP1 = 4; FitP2 = 5;FitP3 = 8
		Fit1 = "Pool1"; Fit2 = "Pool2";Fit3 = "Rh02"
	Endif
end



Macro SetupFit (Alfa, beta, gamma, coop, Pool1, Pool2,rho, delay,rho2)
	Variable Alfa= NumVarOrDefault("alfa", 90)
	Prompt Alfa,"Alfa"
	Variable beta= NumVarOrDefault("beta", 9500)
	Prompt beta,"beta"
	Variable gamma= NumVarOrDefault("gamma", 6000)
	Prompt gamma,"gamma"
	Variable coop= NumVarOrDefault("coop", 0.25)
	Prompt coop,"coop"
	Variable Pool1= NumVarOrDefault("Pool1", 1600)
	Prompt Pool1,"Pool1"
	Variable Pool2= NumVarOrDefault("Pool2", 0)
	Prompt Pool2,"Pool2"
	Variable rho= NumVarOrDefault("rho", -1.2E10)
	Prompt rho,"rho"
	Variable delay= NumVarOrDefault("delay", 2e-4)
	Prompt delay,"delay"
	Variable rho2= NumVarOrDefault("rho2", 0.5)
	Prompt rho2,"rho2"
	 
			//makes a new parameter wave (if ParName is changed) or changes old one
			//stringVariable DefaultparName has to exist (string DefaultParName ="Defaultparams")
			
	$DefaultParName(0) = alfa
	$DefaultParName(1) = beta
	$DefaultParName(2) =  gamma
	$DefaultParName(3) = coop
	$DefaultParName(4) = Pool1
	$DefaultParName(5) = Pool2
	$DefaultParName(6) = rho
	$DefaultParName(7) = delay
	$DefaultParName(8) = rho2
	Variable/G Dt_end, Dflag, DSteady, DBestChi	//For the main Macro
End

Macro SetUpDefaults ()
	string/G Defaultparname = "StartPars"
	string/G Parname = "NotYet"
	String/G Fit1,Fit2,Fit3
	Variable/G Dt_end=0.04, DSteady=0,DFlag=0
	Variable/G FitP1,FitP2,FitP3
	Make/O/N=50  simrate, timewave, stimf
End

Macro Fit5SiteReleaseModel(ParName,t_end,steady,Flag)
	String ParName = StrVarOrDefault("Defaultparname","FittingPars")
	Prompt ParName, "ParName"
	Variable t_end= NumVarOrDefault("Dt_end", 0.04)
	Prompt t_end,"t_end"
	Variable steady= NumVarOrDefault("Dsteady", 10)
	Prompt steady,"steady"
	Variable flag= NumVarOrDefault("Dflag", 1)
	Prompt flag,"flag"
	
	//A 5-binding site release model - see below -  fitted to the release rate as calculated 
	//by the deconvolution from postsynaptic current;  assumes units are vesicles per msec
	//The X-cursor has to be put onto the release rate  in the top window to mark the start of the fitting; 
	//A trace of Ca-current has to be also in this graph named the same as the release rate, except that the 
	//first three letters are missing  and  that it ends ends with "_2nd" (naming convention of Pulse). 
	//The cursor should be placed immediately before the start of the stimulation, e.g. the point of 
	//half rise of the Ca-current. Positive sections of the Ca-current are clipped to zero
	// Fitting is done between the cursor and t_end
	//If flag is	0:	Handfit, all parameters
	//			1:	Fast component:  Poolsize, rho and delay   ( 4,6,7)
	//			2:	Amplitudes:  Pool1, Pool2, and rho  (4,5,6)
	//			3:	Amplitudes:  Pool1, Pool2, and rho2  (4,5,8)
	//  weights:   the first fifth of the  fitting range has 5 time the weigtht  as the rest  (disenabled at the moment!!)
	// initial guesses:  A fixed pattern based on the x-range (Tot) and the y-range (Y_tot)
	// of the data has been programmed (see below). It is readily changed
	
	//We set up the  two component (fast and slow) plus steady state with free parameters:
	//SinglePoolRelease:  1.Size of RRP(A1), 2.Ca-current factor (rho) and 3.)delay: [Ca]= rho* I_Ca(t-delay)
	//2-Poolrelease:	1.Size of RRP(A1)	2. Size of SRP(A2)	and rho2, theo rho for the second component
	//All parameters can be changed by a  handfit 
	
	
	Variable/G  alfa, beta, gamma,coop,Pool1,Pool2,rho, delay,rho2,steady
	Variable/G BestChi= NumVarOrDefault("DBestChi", 1E20)


	IF (CmpStr(ParName,DefaultParName))	//the name of the parameter-wave has changed!
		Make/O/N=10/D/O $ParName
		DefaultParName = ParName	//adopt the new Name as default
		Duplicate/O $ParName BestPars
		AppendToGraph SimRate vs Timewave	//display simulated curve
		ModifyGraph lstyle(SimRate)=2,rgb(SimRate)=(39321,1,31457)
		
	EndIf
	 AssignIndices (flag)
	Print "To be Fitted:  ", Fit1,fit2,fit3
	SetupFit ()
		//this is for updating the defaults
	$Parname[9] = steady
	
	
	DFlag = flag
	Dt_end = t_end
	DSteady = steady
	DBestchi = 1E20

	alfa = $Parname[0]; beta= $Parname[1]; gamma= $Parname[2];coop= $Parname[3];Pool1= $Parname[4]
	Pool2= $Parname[5];rho= $Parname[6]; delay= $Parname[7];rho2= $Parname[8]
	
	string/G  YName, I_CaName
	Variable/G t_start, t_wave, Y_tot,tot, tolerance = 0.0015
	
	Variable/G chi2, p_start, p_end
	YName = CsrWave (A)	//Name of  wave that the cursor is on
	I_CaName = YName
	I_CaName[0,2] = "" 	//take away first 3letters.
	I_CaName += "_2nd" 	//now, this is the Name of corresponding I_Ca trace
	//Print "Traces: ", YName,  I_CaName
	Duplicate/O $YName, Weightf
	Weightf = x				//misuse weightf for calculating t_wave
	t_wave = Weightf[(numpnts(Weightf)-1)]
	t_start = xCsr(A)
	if  (t_end > t_wave)
		Abort  "t_end  later than wave_end"
	endif

	//Print "Parameterwave:  ", ParName,  "Start at: ", t_start, "sec"
	//PrintReleaseParams ($ParName)
	//Print "To be Fitted:  ", Fit1,fit2,fit3

	tot = t_end - t_start
	p_start = x2pnt($YName,t_start); p_end = x2pnt($YName,t_end)
	Wavestats/Q/R=(t_start,t_end)  $YName
	Y_tot = 20*V_max
	Weightf = 1									//equal weights
	//Weightf (t_start,t_start+tot/5) = 5		//first fifth has 5 times the weight

		//initial guesses:
	//A1_In=1600
	//rho_In = -1.2E10
	//delay_In=2e-4/deltax($YName)	//in samples
	//$Parname[7]  /= deltax($YName)	//in samples
	stimf = 0		//initialize the Stimfunction ( prop. to Ca)
	//print  "Delay",$Parname[7]

variable/G  A1_fin, rho_fin,Delay_fin
variable chifinal

	//First,  get estimates for releaserate for the initial guesses;  create waves for these quantities

 //SinglePoolRelRate  ($I_CaName,alfa, beta,gamma,coop,A1_In,p_start,p_end,rho_In,delay_In)
TwoPoolRelRate  ($I_CaName,$Parname,p_start,p_end)
	//Returns the global waves  SimRate and TimeWave as Y- and X- Values of the simulation
	//Now calculate the fitting error (Chi2) 
 chi2 = Linfiterror( $YName, Timewave, SimRate,Weightf,p_start,p_end)

If (flag ==0)		
	print "Manual Fit: Present Chi^2=  ",Chi2,"	Best previous Chi^2:  ",BestChi
	Print "Parameterwave:  ", ParName, "for", YName ,  "	Start at: ", t_start, "sec"
	PrintReleaseParams ($ParName)
	IF (Chi2 <= BestChi)
		DBestChi = Chi2
		BestPars = $ParName
		//Print "new best Chi", BestChi
	EndIf
Else
Print "Initial Chi=  ",Chi2

	//initialize the  Simplex for three free parameters
variable  sumchisquare
Make/N=3/D/O startrates
AssignParsToWave($Parname, startrates,Flag)

Make/N=(4,3)/D/O simplex	//a matrix holding  parameter values
Make/N=4/D/O simplexchis		//
simplexchis = 0

simplex = 0
variable i, j
i=0
	//write the initial parameters and  the 10% changed ones into the simplex  matrix
do
j=0
	do
	simplex[i] [j] =startrates[j]
	j+=1
	while (j<3)
	i +=1
while (i<4)	

i=1
do
simplex [i][i-1] *=1.1
i+=1
while(i<4)
		//get all the chi-squares
i=0
do
AssignMatValuesToPars($Parname, simplex,i,Flag)
TwoPoolRelRate  ($I_CaName,$Parname,p_start,p_end)
simplexchis[i] = Linfiterror( $YName, Timewave, SimRate,Weightf,p_start,p_end)
 i +=1
while(i<4)


//Now we try to go downhill
///determine best, worst and one-but worst vector

variable ilo, ihi, inhi, tempihi, niter, newchi, newnewchi, minimum
wavestats/Q simplexchis
ilo = V_minloc
ihi = V_maxloc
tempihi = simplexchis[ihi] 
simplexchis[ihi] = simplexchis[ilo]
wavestats/Q simplexchis
inhi = V_maxloc
simplexchis[ihi] = tempihi

Make/N=3/D/O tryvector, mirrorplane


//here comes  the main loop
do

//first,do a reflection of the worst point and see if it is any better
// make  reflection vector
mirrorplane = 0

i=0
do
	j=0
	do
		if (i != ihi) 
		mirrorplane[j] += simplex[i][j]
		endif
		j +=1
	while (j<3)
	
	i +=1
while (i<4)
mirrorplane /=3

//make reflection vector
j=0
	do
		//tryvector[j] = mirrorplane[j]-(simplex[ihi][j]-mirrorplane[j])
		tryvector[j] = mirrorplane[j]-1.2*(simplex[ihi][j]-mirrorplane[j])
		 j +=1
	while (j<3)

//is it any better?
//print "reflection of worst point"

AssignWaveToPars( tryvector,$Parname,Flag)
TwoPoolRelRate  ($I_CaName,$Parname,p_start,p_end)

//punt 1

newchi = Linfiterror( $YName, timewave, SimRate,Weightf,p_start,p_end)
//print "partial reflection of worst point", tryvector[0], tryvector[1], tryvector[2], " gives a chi of ",newchi
//if its the best point ever, try to expand
if (newchi<simplexchis[ilo])
		j=0
		do
		simplex[ihi][j] = tryvector[j]
		//tryvector[j] = mirrorplane[j]-2*(simplex[ihi][j]-mirrorplane[j])
		tryvector[j] = mirrorplane[j]-1.4*(simplex[ihi][j]-mirrorplane[j])
		j +=1
		while (j<3)
	simplexchis[ihi]=newchi
	
//see if expanding further improved fit
	//ratedelayca(tryvector[0], tryvector[1], tryvector[2],minca, maxca)
		AssignWaveToPars( tryvector,$Parname,Flag)
		TwoPoolRelRate  ($I_CaName,$Parname,p_start,p_end)
	//SinglePoolRelRate  ($I_CaName,alfa, beta,gamma,coop,tryvector[0],p_start,p_end,tryvector[1],tryvector[2])
	//punt 2
	
	//newnewchi =fiterror($rateca, $ratename, ca, rate)+fiterror($delayca, $delayname, ca, delay)
	newnewchi = Linfiterror( $YName, timewave, SimRate,Weightf,p_start,p_end)
		//print "further expansion to", tryvector[0], tryvector[1], tryvector[2], " gives a chi of ",newnewchi
	if (newnewchi<newchi)
		j=0
			do
			simplex[ihi][j] = tryvector[j]
			j +=1
			while (j<3)
		simplexchis[ihi]=newnewchi
	endif

// if its not the best, see if it better than worst
else
if (newchi<simplexchis[inhi])
		j=0
		do
		simplex[ihi][j] = tryvector[j]
		j +=1		
		while (j<3)
		simplexchis[ihi] = newchi
	else	
//if it's better than worst but worse than second worst, try to contract
	if ((newchi>=simplexchis[inhi]))
		j=0
		do
		//tryvector[j] = mirrorplane[j]+0.5*(simplex[ihi][j]-mirrorplane[j])
		tryvector[j] = mirrorplane[j]-0.6*(simplex[ihi][j]-mirrorplane[j])
		j +=1
		while (j<3)
	
	//see if this helped	
	//ratedelayca(tryvector[0], tryvector[1], tryvector[2],minca, maxca)
		AssignWaveToPars( tryvector,$Parname,Flag)
		TwoPoolRelRate  ($I_CaName,$Parname,p_start,p_end)
	//SinglePoolRelRate  ($I_CaName,alfa, beta,gamma,coop,tryvector[0],p_start,p_end,tryvector[1],tryvector[2])
	//punt 3
	
	//newnewchi = fiterror($rateca, $ratename, ca, rate)+fiterror($delayca, $delayname, ca, delay)
	newnewchi = Linfiterror( $YName, timewave, SimRate,Weightf,p_start,p_end)
	//print "contraction to", tryvector[0], tryvector[1], tryvector[2], " gives a chi of ",newnewchi
	if (newnewchi<newchi)
		j=0
		do
		simplex[ihi][j] = tryvector[j]
		j +=1		
		while (j<3)
		simplexchis[ihi]=newnewchi
	
	// contracting did not help, so we do the multiple contraction to the best point and reevaluate all the points
	else	
		i=0

		do
			j=0
			do
			minimum = simplex[ilo] [j]
			simplex[i] [j] /=2
			simplex[i] [j] += minimum/2
			j+=1
			while (j<3)
		i +=1
		while (i<4)
		break
		i=0
		do
		
		//ratedelayca(simplex[i][0], simplex[i][1], simplex[i][2],minca, maxca)
		AssignWaveToPars( tryvector,$Parname,Flag)
		TwoPoolRelRate  ($I_CaName,$Parname,p_start,p_end)
		//SinglePoolRelRate  ($I_CaName,alfa, beta,gamma,coop,simplex[i][0],p_start,p_end,simplex[i][1],simplex[i][2])
		//punt 4
		
		//simplexchis[i] = fiterror($rateca, $ratename, ca, rate)+fiterror($delayca, $delayname, ca, delay)
		simplexchis[i] = Linfiterror( $YName, timewave, SimRate,Weightf,p_start,p_end)
		//print "multiple contraction to", tryvector[0], tryvector[1], tryvector[2], " gives a chi of "simplexchis[i] 
		i +=1
		while(i<4)
 	endif
 endif	
endif
endif 

wavestats/Q simplexchis
ilo = V_minloc
ihi = V_maxloc
tempihi = simplexchis[ihi] 
simplexchis[ihi] = simplexchis[ilo]
wavestats/Q simplexchis
inhi = V_maxloc
simplexchis[ihi] = tempihi
print  Niter, simplexchis[ilo], tempihi
Niter +=1
//print 2*(simplexchis[ihi]- simplexchis[ilo])/(simplexchis[ihi]+ simplexchis[ilo])
//print simplex[ilo][0], simplex[ilo][1], simplex[ilo][2]
while ((2*(simplexchis[ihi]- simplexchis[ilo])/(simplexchis[ihi]+ simplexchis[ilo])) >tolerance)
	//the end of the main loop; the relative difference between the highest and lowest chi-square has 
	//to be smaller than 'tolerance

	//Now get the numbers for the best value
//ratedelayca(simplex[ilo][0], simplex[ilo][1], simplex[ilo][2], minca, maxca)

AssignMatValuesToPars($Parname, simplex,ilo,Flag)
TwoPoolRelRate  ($I_CaName,$Parname,p_start,p_end)

// strange, why did I calculate this for  the i-values and not the ilo - ones
//SinglePoolRelRate  ($I_CaName,alfa, beta,gamma,coop,simplex[i][0],p_start,p_end,simplex[i][1],simplex[i][2])

//print simplex[ilo][0], simplex[ilo][1], simplex[ilo][2]
	Print "Parameterwave:  ", ParName, "for", YName ,  "	Start at: ", t_start, "sec"
	Print "Final Parameters for Chi^2:", simplexchis[ilo]
	PrintReleaseParams ($ParName)
//adopt new  values as defaults:
AssignFitVariables (ParName)
//alfa = Parwave[0]; beta = Parwave[1]; gamma = Parwave[2]; coop = Parwave[3]
//Pool1 = Parwave[4]; Pool2 = Parwave[5]; rho = Parwave[6]; delay = Parwave[7]
//rho2 = Parwave[8]; steady =  Parwave[9]
	
ENDIF	// (IF (Flag == 0)

END

Macro AssignFitVariables (ParWave)
	string Parwave
	alfa = $Parwave[0]; beta = $Parwave[1]; gamma = $Parwave[2]; coop = $Parwave[3]
	Pool1 = $Parwave[4]; Pool2 = $Parwave[5]; rho = $Parwave[6]; delay = $Parwave[7]
	rho2 = $Parwave[8]; steady = $Parwave[9]
End
	
	

Function PrintReleaseParams (Parwave)
	wave Parwave

	SVAR Fit1 = Fit1
	SVAR fit2 = Fit2
	SVAR fit3 = Fit3
	//prints the params of the 5Ca-model and indicates which ones were fitted
	//Parwave: 0,1,2,3:  	alfa, beta, gamma, coop
	//			4,5:		Pool sizes
	//			6,7,8: 		rho,  delay, and   rho2
	Print  "Alfa, beta, gamma, coop:	",Parwave[0],Parwave[1],Parwave[2],Parwave[3]
	Print  "Pool1, Pool2:		(4,5)	",Parwave[4],Parwave[5],"   vesicles"
	Print  "rho, delay,rho2:	(6,7,8)",Parwave[6],1000*Parwave[7],Parwave[8],"uM/A , msec, uM/A"
	//Print "Fitted:  ", Fit1,fit2,fit3
	Print "steady:	", Parwave[9]
End


function cadepend1 (calWave,alfa, beta,gamma, coop,Poolsize,p_start,p_end)
Wave calWave
variable  alfa , beta , gamma , coop,Poolsize,p_start,p_end 
	//This is a Runge-Kutta integration  with step size control according to 'Numerical Recipes'
	//calculates time-courses for a  5-site model

variable steps =20000	//we do not know how long the traces will be; so we make them long enough 
						//and trim them later
Make/N=(steps)/D/O bound0,bound1,bound2,bound3, bound4,bound5, exo,timewave
						//make waves for the different states  and time wave and initialitze variables
bound0[0]=Poolsize
bound1[0] =0
bound2[0] = 0
bound3[0] = 0
bound4[0] = 0
bound5[0] = 0
exo[0]=0
timewave[0]=pnt2x(calWave,p_start)
variable stepsize = 0.00005
variable i=1


variable a2 = 0.2, a3 = 0.3, a4 = 0.6, a5 = 1, a6=0.875, b21=0.2, b31=0.075, b32=0.225, b41=0.3, b42=-0.9, b43=1.2, b51=-11/54, b52=2.5, b53=-70/27, b54=35/27, b61=1631/55296
variable b62=175/512, b63=575/13824, b64=44275/110592, b65=253/4096, c1=37/378, c3=250/621, c4=125/594, c6=512/1771
variable bound01, bound02, bound03, bound04, bound11, bound12, bound13, bound14,  bound21, bound22, bound23, bound24,  bound31, bound32, bound33, bound34
variable bound05, bound06, bound15, bound16, bound25, bound26, bound35, bound36, bound41, bound42, bound43, bound44, bound45, bound46,bound51, bound52, bound53, bound54, bound55, bound56
variable dc1, dc3, dc4, dc5, dc6

variable maxfout = 0.1
dc1 = c1-2825/27648
dc3 = c3-18575/48384
dc5 = -277/14336
dc6 = c6-0.25 

silent 1; pauseupdate
Make/N=6/D/O error
Variable  calcium

	//now, do the Runge-Kutta

do
calcium = calWave(timewave[i-1])

timewave[i]	= timewave[i-1]+stepsize
bound01	=	(	-5*alfa*calcium		*bound0[i-1]		+	beta					*bound1[i-1]																						)*stepsize
bound11	=	(	5*alfa*calcium		*bound0[i-1]		-	(beta+4*alfa*calcium)	*bound1[i-1]	+	(2*beta*coop)				*bound2[i-1]											)*stepsize
bound21	=	(										+	4*alfa*calcium		*bound1[i-1]	-	(2*beta*coop+3*alfa*calcium)	*bound2[i-1]	+	(3*beta*coop^2)				*bound3[i-1]	)*stepsize
bound31 =	(																			+	3*alfa*calcium				*bound2[i-1]	-	(3*beta*coop^2+2*alfa*calcium)	*bound3[i-1]		+	(4*beta*coop^3) 				* bound4[i-1]	)*stepsize	
bound41 =	(																														+	2*alfa*calcium					*bound3[i-1]	-	(4*beta*coop^3+alfa*calcium) 	* bound4[i-1]		+	5*beta*coop^4			*	bound5[i-1])*stepsize	
bound51 =	(																																										+	alfa*calcium 					* bound4[i-1]		-	(5*beta*coop^4+gamma)	*	bound5[i-1])*stepsize	


bound02	=	(	-5*alfa*calcium		*(bound0[i-1]+b21*bound01)		+	beta					*(bound1[i-1]+b21*bound11)																															)*stepsize
bound12	=	(	5*alfa*calcium		*(bound0[i-1]+b21*bound01)		-	(beta+4*alfa*calcium)	*(bound1[i-1]+b21*bound11)		+	(2*beta*coop)				*(bound2[i-1]+b21*bound21)															)*stepsize
bound22	=	(														+	4*alfa*calcium		*(bound1[i-1]+b21*bound11)		-	(2*beta*coop+3*alfa*calcium)	*(bound2[i-1]+b21*bound21)	+	(3*beta*coop^2)					*(bound3[i-1]+b21*bound31)	)*stepsize
bound32 =	(																												+	3*alfa*calcium				*(bound2[i-1]+b21*bound21)	-	(3*beta*coop^2 +2*alfa*calcium)	*(bound3[i-1]+b21*bound31)	+	(4*beta*coop^3) 				* (bound4[i-1]	+b21*bound41)	)*stepsize	
bound42 =	(																																											+	2*alfa*calcium					*(bound3[i-1]+b21*bound31)	-	(4*beta*coop^3+alfa*calcium) 	* (bound4[i-1]	+b21*bound41)		+	5*beta*coop^4			*	(bound5[i-1]+b21*bound51))*stepsize	
bound52 =	(																																																											+	alfa*calcium 					* (bound4[i-1]	+b21*bound41)		-	(5*beta*coop^4+gamma)	*	(bound5[i-1]+b21*bound51))*stepsize	

bound03	=	(	-5*alfa*calcium		*(bound0[i-1]+b31*bound01+b32*bound02)		+	beta					*(bound1[i-1]+b31*bound11+b32*bound12)																																						)*stepsize
bound13	=	(	5*alfa*calcium		*(bound0[i-1]+b31*bound01+b32*bound02)		-	(beta+4*alfa*calcium)	*(bound1[i-1]+b31*bound11+b32*bound12)		+	(2*beta*coop)				*(bound2[i-1]+b31*bound21+b32*bound22)																			)*stepsize
bound23	=	(																	+	4*alfa*calcium		*(bound1[i-1]+b31*bound11+b32*bound12)		-	(2*beta*coop+3*alfa*calcium)	*(bound2[i-1]+b31*bound21+b32*bound22)		+	(3*beta*coop^2)			*(bound3[i-1]+b31*bound31+b32*bound32)	)*stepsize
bound33 =	(																																		+	3*alfa*calcium				*(bound2[i-1]+b31*bound21+b32*bound22)	-	(3*beta*coop^2 +2*alfa*calcium)	*(bound3[i-1]+b31*bound31+b32*bound32)	+	(4*beta*coop^3) 				* (bound4[i-1]	+b31*bound41+b32*bound42))	*stepsize	
bound43 =	(																																																					+	2*alfa*calcium				*(bound3[i-1]+b31*bound31+b32*bound32)	-	(4*beta*coop^3+alfa*calcium) 	* (bound4[i-1]	+b31*bound41+b32*bound42)		+	5*beta*coop^4			*	(bound5[i-1]+b31*bound51+b32*bound52))	*stepsize	
bound53 =	(																																																																							+	alfa*calcium 					* (bound4[i-1]	+b31*bound41+b32*bound42)		-	(5*beta*coop^4+gamma)	*	(bound5[i-1]+b31*bound51+b32*bound52))*stepsize	

bound04	=	(	-5*alfa*calcium		*(bound0[i-1]+b41*bound01+b42*bound02+b43*bound03)		+	beta					*(bound1[i-1]+b41*bound11+b42*bound12+b43*bound13)																																										)*stepsize
bound14	=	(	5*alfa*calcium		*(bound0[i-1]+b41*bound01+b42*bound02+b43*bound03)		-	(beta+4*alfa*calcium)	*(bound1[i-1]+b41*bound11+b42*bound12+b43*bound13)	+	(2*beta*coop)				*(bound2[i-1]+b41*bound21+b42*bound22+b43*bound23)																					)*stepsize
bound24	=	(																				+	4*alfa*calcium		*(bound1[i-1]+b41*bound11+b42*bound12+b43*bound13)	-	(2*beta*coop+3*alfa*calcium)	*(bound2[i-1]+b41*bound21+b42*bound22+b43*bound23)		+	(3*beta*coop^2)					*(bound3[i-1]+b41*bound31+b42*bound32+b43*bound33)	)*stepsize
bound34 =	(																																							+	3*alfa*calcium				*(bound2[i-1]+b41*bound21+b42*bound22+b43*bound23)		-	(3*beta*coop^2 +2*alfa*calcium)	*(bound3[i-1]+b41*bound31+b42*bound32+b43*bound33)	+	(4*beta*coop^3)				*(bound4[i-1]+b41*bound41+b42*bound42+b43*bound43) 	)*stepsize	
bound44 =	(																																																													+	2*alfa*calcium					*(bound3[i-1]+b41*bound31+b42*bound32+b43*bound33)	-	(4*beta*coop^3+alfa*calcium) 	*(bound4[i-1]+b41*bound41+b42*bound42+b43*bound43)		+	5*beta*coop^4			*	(bound5[i-1]++b41*bound51+b42*bound52+b43*bound53))*stepsize	
bound54 =	(																																																																																			+	alfa*calcium 					*(bound4[i-1]+b41*bound41+b42*bound42+b43*bound43)		-	(5*beta*coop^4+gamma)	*	(bound5[i-1]++b41*bound51+b42*bound52+b43*bound53))*stepsize	

bound05	=	(	-5*alfa*calcium		*(bound0[i-1]+b51*bound01+b52*bound02+b53*bound03+b54*bound04)		+	beta					*(bound1[i-1]+b51*bound11+b52*bound12+b53*bound13+b54*bound14)																				)*stepsize
bound15	=	(	5*alfa*calcium		*(bound0[i-1]+b51*bound01+b52*bound02+b53*bound03+b54*bound04)		-	(beta+4*alfa*calcium)	*(bound1[i-1]+b51*bound11+b52*bound12+b53*bound13+b54*bound14)		+	(2*beta*coop)				*(bound2[i-1]+b51*bound21+b52*bound22+b53*bound23+b54*bound24)										)*stepsize
bound25	=	(																							+	4*alfa*calcium		*(bound1[i-1]+b51*bound11+b52*bound12+b53*bound13+b54*bound14)		-	(2*beta*coop+3*alfa*calcium)	*(bound2[i-1]+b51*bound21+b52*bound22+b53*bound23+b54*bound24)	+	(3*beta*coop^2)					*(bound3[i-1]+b51*bound31+b52*bound32+b53*bound33+b54*bound34)	)*stepsize
bound35 =	(																																														+	3*alfa*calcium				*(bound2[i-1]+b51*bound21+b52*bound22+b53*bound23+b54*bound24)	-	(3*beta*coop^2 +2*alfa*calcium)	*(bound3[i-1]+b51*bound31+b52*bound32+b53*bound33+b54*bound34)	+ 	(4*beta*coop^3+alfa*calcium)	* (bound4[i-1]+b51*bound41+b52*bound42+b53*bound43+b54*bound44)	)*stepsize	
bound45 =	(																																																																						+	2*alfa*calcium					*(bound3[i-1]+b51*bound31+b52*bound32+b53*bound33+b54*bound34)	-	(4*beta*coop^3+alfa*calcium) 	* (bound4[i-1]+b51*bound41+b52*bound42+b53*bound43+b54*bound44)		+	5*beta*coop^4			*	(bound5[i-1]+b51*bound51+b52*bound52+b53*bound53+b54*bound54))*stepsize	
bound55 =	(																																																																																															+	alfa*calcium 					* (bound4[i-1]+b51*bound41+b52*bound42+b53*bound43+b54*bound44)		-	(5*beta*coop^4+gamma)	*	(bound5[i-1]+b51*bound51+b52*bound52+b53*bound53+b54*bound54))*stepsize	

bound06	=	(	-5*alfa*calcium		*(bound0[i-1]+b61*bound01+b62*bound02+b63*bound03+b64*bound04+b65*bound05)		+	beta					*(bound1[i-1]+b61*bound11+b62*bound12+b63*bound13+b64*bound14+b65*bound15)																				)*stepsize
bound16	=	(	5*alfa*calcium		*(bound0[i-1]+b61*bound01+b62*bound02+b63*bound03+b64*bound04+b65*bound05)		-	(beta+4*alfa*calcium)	*(bound1[i-1]+b61*bound11+b62*bound12+b63*bound13+b64*bound14+b65*bound15)		+	(2*beta*coop)				*(bound2[i-1]+b61*bound21+b62*bound22+b63*bound23+b64*bound24+b65*bound25)										)*stepsize
bound26	=	(																										+	4*alfa*calcium		*(bound1[i-1]+b61*bound11+b62*bound12+b63*bound13+b64*bound14+b65*bound15)		-	(2*beta*coop+3*alfa*calcium)	*(bound2[i-1]+b61*bound21+b62*bound22+b63*bound23+b64*bound24+b65*bound25)		+	(3*beta*coop^2)					*(bound3[i-1]+b61*bound31+b62*bound32+b63*bound33+b64*bound34+b65*bound35)	)*stepsize
bound36 =	(																																																				+	3*alfa*calcium				*(bound2[i-1]+b61*bound21+b62*bound22+b63*bound23+b64*bound24+b65*bound25)		-	(3*beta*coop^2 +2*alfa*calcium)	*(bound3[i-1]+b61*bound31+b62*bound32+b63*bound33+b64*bound34+b65*bound35)+	(4*beta*coop^3) 				*(bound4[i-1]+b61*bound41+b62*bound42+b63*bound43+b64*bound44+b65*bound45)	)*stepsize	
bound46 =	(																																																																																+	2*alfa*calcium					*(bound3[i-1]+b61*bound31+b62*bound32+b63*bound33+b64*bound34+b65*bound35)	-	(4*beta*coop^3+alfa*calcium) 	*(bound4[i-1]+b61*bound41+b62*bound42+b63*bound43+b64*bound44+b65*bound45)		+	5*beta*coop^4			*	(bound5[i-1]+b61*bound51+b62*bound52+b63*bound53+b64*bound54+b65*bound55))*stepsize	
bound56 =	(																																																																																																												+	alfa*calcium 					*(bound4[i-1]+b61*bound41+b62*bound42+b63*bound43+b64*bound44+b65*bound45)		-	(5*beta*coop^4+gamma)	*	(bound5[i-1]+b61*bound51+b62*bound52+b63*bound53+b64*bound54+b65*bound55))*stepsize	

	
bound0[i]	=	bound0[i-1]	+	c1*bound01 + c3*bound03 + c4*bound04 + c6 *bound06
bound1[i]	=	bound1[i-1]	+	c1*bound11 + c3*bound13 + c4*bound14 + c6 *bound16
bound2[i]	=	bound2[i-1]	+	c1*bound21 + c3*bound23 + c4*bound24 + c6 *bound26
bound3[i]	=	bound3[i-1]	+	c1*bound31 + c3*bound33 + c4*bound34 + c6 *bound36
bound4[i]	=	bound4[i-1]	+	c1*bound41 + c3*bound43 + c4*bound44 + c6 *bound46
bound5[i]	=	bound5[i-1]	+	c1*bound51 + c3*bound53 + c4*bound54 + c6 *bound56
exo[i]= Poolsize-bound0[i]-bound1[i]-bound2[i]-bound3[i]-bound4[i]-bound5[i]

error[0] 	= 	abs(dc1	*	bound01			+	dc3		*	bound03		+	dc4	*	bound04		+	dc5	*	bound05	+	dc6	*	bound06)
error[1]	= 	abs(dc1	*	bound11			+	dc3		*	bound13		+	dc4	*	bound14		+	dc5	*	bound15	+	dc6	*	bound16)
error[2] 	= 	abs(dc1	*	bound21			+	dc3		*	bound23		+	dc4	*	bound24		+	dc5	*	bound25	+	dc6	*	bound26)
error[3] 	=	abs(dc1	*	bound31			+	dc3		*	bound33		+	dc4	*	bound34		+	dc5	*	bound35	+	dc6	*	bound36	)		
error[4] 	=	abs(dc1	*	bound41			+	dc3		*	bound43		+	dc4	*	bound44		+	dc5	*	bound45	+	dc6	*	bound46	)					
error[5] 	=	abs(dc1	*	bound51			+	dc3		*	bound53		+	dc4	*	bound54		+	dc5	*	bound55	+	dc6	*	bound56	)					
//print error[0], error[1],error[2],error[3]

wavestats/Q error
if  (V_max== 0)
	V_max = 0.001
endif
if (V_max<=maxfout)
	//print "oude stepsize", stepsize, " max-fout", V_max
	stepsize *=0.9*abs((maxfout/V_max)^.2)				
	//print "nieuwe stepsize", stepsize	
	i+=1
else
		//print "else:  oude stepsize", stepsize, " max-fout", V_max 
	stepsize *=0.9*abs((maxfout/V_max)^.25)
//	print "nieuwe stepsize", stepsize	
endif

while ((i<= steps) %& (timewave[i-1]<pnt2x(calWave,p_end)))	//end of  Runge-Kutta

	//now, trim the curves (delete excess points)
DeletePoints i,20000-i, timewave,exo,bound3,bound2,bound1,bound0

end




Function clip1 (distrib, thresh) 	//clip the function to a minimum value of 'thresh'
	wave  distrib	//wave to be clipped
	Variable Thresh	//clipping threshold

	Variable i=0, lastpoint = Numpnts(distrib)
	do
		IF 	(distrib[i] < thresh)
		distrib[i] = thresh
	endif
	//print i, distrib[i] 
	i += 1
	while (i < lastpoint)
end



function Linfiterror( datay, simulx, simuly,weight,p_start,p_end)
wave datay, simulx, simuly,weight
Variable  p_start,p_end
	//This function calculates the fit-error as product of the sqare-deviation with a weight function
	//y-data are equally spaced ; simulation results are interpolatedinterpolated between calculated points

variable chi=0, teller

teller = p_start
do
//print datax[teller], datay[teller], interp(datax[teller],simulx, simuly),  (log((datay[teller]/interp(datax[teller],simulx, simuly))))^2
chi += ((datay[teller]-interp(pnt2x(datay,teller),simulx, simuly))*weight[teller])^2

teller+=1
while (teller<p_end)
//print "chi" ,chi
return chi
end

